# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''StaticRst objects (also known as "flatpages")  provide for free-form page
layouts.
'''


import os

from firmant import decorators
from firmant import parsers


class StaticRstObject(parsers.RstParsedObject):
    '''A static object that contains title, content, and path information.

    The difference between this class and
    :class:`firmant.parsers.static.StaticObject` is this class is parsed from
    a restructured text document while the other is only from filesystem
    information.

    Attributes of :class:`StaticRstObject`:

        title
           The title from the parsed restructured text document.

        path
           The path relative to the root where the static objects are stored.

        content
           The content of the restructured text document.  This includes the
           title and subtitle components from the docutils writer.

    .. doctest::

       >>> StaticRstObject(path='projects/firmant')
       staticrst_obj<projects/firmant>

    '''

    # pylint: disable-msg=R0903

    __slots__ = ['path']

    _pubparts = [('content', 'html_body')
                   ,('title', 'title')
                   ]

    def __repr__(self):
        return 'staticrst_obj<%s>' % getattr(self, 'path', None)

    @property
    def _attributes(self):
        '''Attributes that identify a static object:

            path
               A path that describes the object relative to the input/output
               directories.

        .. doctest::

           >>> pprint(StaticRstObject(path='about/projects/firmant')._attributes)
           {'path': 'about/projects/firmant'}

        '''
        # pylint: disable-msg=E1101
        return {'path': self.path}


class StaticRstParser(parsers.RstParser):
    '''Create a static page from reStructured Text.

    .. doctest::
       :hide:

       >>> environment['log'] = get_logger()

    .. doctest::

       >>> srp = StaticRstParser(environment, objects)
       >>> environment, objects, (parser,) = srp(environment, objects)
       >>> pprint(parser(environment, objects)) #doctest: +ELLIPSIS
       ({...},
        {'staticrst': [staticrst_obj<about>,
                       staticrst_obj<empty>,
                       staticrst_obj<links>]},
        [])

    '''

    type = 'staticrst'
    paths = '.*\.rst'
    cls = StaticRstObject

    @decorators.in_environment('settings')
    def root(self, environment):
        '''The directory under which all staticrst objects reside.
        '''
        settings = environment['settings']
        return os.path.join(settings.CONTENT_ROOT, settings.STATIC_RST_SUBDIR)

    def rstparse(self, environment, objects, path, pieces):
        '''Use the parsed rst document to construct the necessary objects.
        '''
        attrs = {}
        attrs['path'] = unicode(path[:-4])
        attrs['_pub'] = pieces['pub']
        objects[self.type].append(self.cls(**attrs))


def _setup(test):
    '''Setup the tests.
    '''
    from pysettings.settings import Settings
    from firmant import routing
    test.globs['settings'] = Settings({'CONTENT_ROOT': 'testdata/pristine'
                                      ,'STATIC_RST_SUBDIR': 'flat'
                                      })
    urlmapper = routing.URLMapper('/path/to/output/dir', 'http://testurl/', [])
    test.globs['environment'] = {'settings': test.globs['settings']
                                ,'urlmapper': urlmapper
                                }
    test.globs['objects'] = {}



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Utilities for manipulating paths/filesystems.
'''


import os
import re
import sys


def cat(path, out=None):
    r'''Write the contents of file ``path`` to ``out``.

    .. doctest::
       :hide:

       >>> import sys

    .. doctest::

       >>> cat('testdata/settings/empty.py') #doctest: +ELLIPSIS
       # Copyright (c) 2010, Robert Escriva
       ...
       # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    '''
    fil = open(path)
    if out is None:
        out = sys.stdout
    for line in fil:
        print >> out, line,
    fil.close()


def create_or_truncate(path):
    r'''Open ``path``, creating it or directories as necessary.

    In this example, the file `path` is `foo/bar/baz/quux` relative to a
    temporary directory.  It is opened once for reading and the string ``THIS IS
    THE FIRST TIME`` is written to the file.  The file is closed and reopened
    using create_or_truncate, and written again with ``THIS IS THE SECOND
    TIME``.

    .. doctest::
       :hide:

       >>> import tempfile
       >>> root = tempfile.mkdtemp()

    .. doctest::

       >>> path = os.path.join(root, 'foo/bar/baz/quux')

       >>> f = create_or_truncate(path)
       >>> f.write('THIS IS THE FIRST TIME\n')
       >>> f.flush() and f.close()

       >>> cat(path)
       THIS IS THE FIRST TIME

       >>> f2 = create_or_truncate(path)
       >>> f2.write('THIS IS THE SECOND TIME\n')
       >>> f2.flush() and f2.close()

       >>> cat(path)
       THIS IS THE SECOND TIME

    .. doctest::
       :hide:

       >>> # Cleanup after ourselves.
       >>> import shutil
       >>> shutil.rmtree(root)

    Errors from the underlying operating system will be raised.

    .. doctest::

       >>> create_or_truncate('/tmp')
       Traceback (most recent call last):
       IOError: [Errno 21] Is a directory: '/tmp'

    .. doctest::
       :hide:

       >>> from minimock import restore
       >>> old_makedirs = os.makedirs
       >>> os.makedirs = Mock('makedirs')
       >>> os.makedirs.mock_raises = OSError(21)
       >>> os.makedirs.mock_raises.errno = 21
       >>> create_or_truncate('/')
       Traceback (most recent call last):
       OSError: 21
       >>> os.makedirs = old_makedirs

    '''
    par = os.path.dirname(path)
    if par != '':
        try:
            os.makedirs(par)
        except OSError, ex:
            if ex.errno != 17:
                raise
    return open(path, 'w+')


def recursive_listdir(root, matches=None, files_only=True):
    '''Provide a list of all files in a directory and its subdirectories.

    .. doctest::

       >>> pprint(sorted(recursive_listdir('testdata/pristine/posts')))
       ['2009-12-31-party.rst',
        '2010-01-01-newyear.rst',
        '2010-02-01-newmonth.rst',
        '2010-02-02-newday.rst',
        '2010-02-02-newday2.rst']

    An optional regex string `matches` may be specified and will be compared to
    the :func:`os.path.basename` of each entry under path.

    .. doctest::

       >>> pprint(sorted(recursive_listdir('testdata/pristine',
       ...                                 matches='.*\.rst$')))
       ['feeds/bar.rst',
        'feeds/baz.rst',
        'feeds/foo.rst',
        'feeds/quux.rst',
        'flat/about.rst',
        'flat/empty.rst',
        'flat/links.rst',
        'posts/2009-12-31-party.rst',
        'posts/2010-01-01-newyear.rst',
        'posts/2010-02-01-newmonth.rst',
        'posts/2010-02-02-newday.rst',
        'posts/2010-02-02-newday2.rst',
        'tags/bar.rst',
        'tags/baz.rst',
        'tags/foo.rst',
        'tags/quux.rst']

    '''
    ret = []
    for relroot, dirs, files in os.walk(root):
        if files_only:
            dirs = []
        for ent in dirs + files:
            path = os.path.join(relroot, ent)
            if path.startswith(root):
                path = path[len(root):]
            else:
                raise RuntimeError(_('`path` expected to exist under `root`'))
            if path.startswith('/'):
                path = path[1:]
            if not matches or re.match(matches, path):
                ret.append(path)
    return ret



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Paginate or group objects according to certain properties.
'''


class Paginated(object):
    '''An object representing keys in paginated lists.

    The key used for each of the lists is stored in the properties ``prev``,
    ``cur``, ``next``.  Instances are just nice containers in place of
    3-tuples::

    .. doctest::

       >>> p = Paginated(1, 2, 3)
       >>> p
       Paginated(1, 2, 3)
       >>> p.prev
       1
       >>> p.cur
       2
       >>> p.next
       3

    '''

    def __init__(self, prev, cur, nex):
        self._prev = prev
        self._cur = cur
        self._next = nex

    @property
    def prev(self):
        '''The key of the previous list.
        '''
        return self._prev

    @property
    def cur(self):
        '''The key of the current list.
        '''
        return self._cur

    @property
    def next(self):
        '''The key of the next list.
        '''
        return self._next

    def __repr__(self):
        '''A more human-friendly form for representing paginated objects.
        '''
        return 'Paginated(%s, %s, %s)' % \
               (str(self._prev), str(self._cur), str(self._next))


def split_list(key_func, obj_list):
    '''Split `obj_list` at boundaries determined by key_func.

    At each point the return value of `key_func(obj)` changes, split `obj_list`.
    The return value is a list of these split lists.

    The function's behavior is undefined if `key_func` ever returns `None` (If
    you're tempted to return `None` anyways, read this statement as "Your code
    will break if `key_func` ever returns `None`").

    .. doctest::

       >>> def parity(x):
       ...     return x % 2
       >>> pprint(split_list(parity, [1, 3, 5, 2, 4, 7, 8, 9, 10]))
       [([1, 3, 5], Paginated(None, 1, 0)),
        ([2, 4], Paginated(1, 0, 1)),
        ([7], Paginated(0, 1, 0)),
        ([8], Paginated(1, 0, 1)),
        ([9], Paginated(0, 1, 0)),
        ([10], Paginated(1, 0, None))]
       >>> split_list(parity, [])
       []
       >>> split_list(parity, [1])
       [([1], Paginated(None, 1, None))]

    '''
    cur_list = obj_list
    nex_list = cur_list[1:] + [None]

    kprev = None

    ret = []
    acc = []
    for cur, nex in zip(cur_list, nex_list):
        kcur = cur and key_func(cur)
        knex = nex and key_func(nex)
        acc.append(cur)
        if kcur != knex:
            ret.append((acc, Paginated(kprev, kcur, knex)))
            kprev = kcur
            acc = []
    return ret


def paginate(num_per_page, obj_list):
    '''Break `obj_list` into lists of at most `num_per_page` objects.

        >>> pprint(paginate(1, [1, 2, 3, 4, 5, 6, 7]))
        [([1], Paginated(None, 1, 2)),
         ([2], Paginated(1, 2, 3)),
         ([3], Paginated(2, 3, 4)),
         ([4], Paginated(3, 4, 5)),
         ([5], Paginated(4, 5, 6)),
         ([6], Paginated(5, 6, 7)),
         ([7], Paginated(6, 7, None))]
        >>> pprint(paginate(2, [1, 2, 3, 4, 5, 6, 7]))
        [([1, 2], Paginated(None, 1, 2)),
         ([3, 4], Paginated(1, 2, 3)),
         ([5, 6], Paginated(2, 3, 4)),
         ([7], Paginated(3, 4, None))]
        >>> paginate(2, [])
        []
        >>> paginate(2, [1])
        [([1], Paginated(None, 1, None))]

    '''
    num_pages = (len(obj_list) + num_per_page - 1) / num_per_page

    prev_list = [None] + range(1, num_pages)
    cur_list  = range(1, num_pages + 1)
    nex_list  = range(2, num_pages + 1) + [None]

    ret = []
    for prev, cur, nex in zip(prev_list, cur_list, nex_list):
        begin = (cur - 1) * num_per_page
        end   = begin + num_per_page
        objs  = obj_list[begin:end]
        ret.append((objs, Paginated(prev, cur, nex)))
    return ret


def split_paginate(num_per_page, key_func, obj_list):
    '''Split `obj_list` with `split_list`; then use `paginate`.

    The lists will be split according to `split_list`.  Each of the
    resulting lists will then be passed to `paginate`.  The result will be
    a nested datastructure that is best explained by example.

    .. doctest::

       >>> def parity(x):
       ...     return x % 2
       >>> pprint(split_paginate(2, parity, [1, 3, 5, 7, 9, 2, 4, 7, 9, 10]))
       [([1, 3], Paginated(None, 1, 0), Paginated(None, 1, 2)),
        ([5, 7], Paginated(None, 1, 0), Paginated(1, 2, 3)),
        ([9], Paginated(None, 1, 0), Paginated(2, 3, None)),
        ([2, 4], Paginated(1, 0, 1), Paginated(None, 1, None)),
        ([7, 9], Paginated(0, 1, 0), Paginated(None, 1, None)),
        ([10], Paginated(1, 0, None), Paginated(None, 1, None))]
       >>> split_paginate(2, parity, [])
       []
       >>> split_paginate(2, parity, [1])
       [([1], Paginated(None, 1, None), Paginated(None, 1, None))]
       >>> pprint(split_paginate(2, parity, [1, 2]))
       [([1], Paginated(None, 1, 0), Paginated(None, 1, None)),
        ([2], Paginated(1, 0, None), Paginated(None, 1, None))]

    '''
    ret = []
    for split_obj, split_key in split_list(key_func, obj_list):
        for page_list, page_key in paginate(num_per_page, split_obj):
            ret.append((page_list, split_key, page_key))
    return ret



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Manipulate post objects for Firmant.
'''


import datetime
import os

import pytz

from firmant import decorators
from firmant import parsers
from firmant.utils import strptime


class Post(parsers.RstParsedObject):
    '''A post is piece of content identified by a date of publication and a
    slug.

    Attributes of :class:`Post`:

        slug
           A unique string that identifies the post.

        published
           The date and time of publication of the post.

        title
           A longer title used for identifying the post to the user.

        author
           The person who authored the post.

        copyright
           The copyright of the post.  Absence of a copyright implies all rights
           are reserved (except in the case that a post without a copyright
           belongs to a feed with a copyright).

        content
           The content of the restructured text document.  This does not include
           the title information.

        updated
           The time at which the post was last updated (defaults to the time of
           publication).

        tags
           A list of cross-referenced tags.  This will contain only slugs until
           cross-referencing happens at a later point in time.

        feeds
           A list of cross-referenced feeds.  This will contain only slugs until
           cross-referencing happens at a later point in time.

    .. note::

       All string attributes except `slug` may be ``''``.  Posts will be ``[]``
       until the cross-referencing chunk happens.

    .. doctest::

       >>> from pytz import utc
       >>> from datetime import datetime
       >>> Post(published=utc.localize(datetime(2010, 1, 1)), slug='foopost')
       Post(2010-01-01-foopost)

    '''

    # pylint: disable-msg=R0903

    __slots__ = ['slug', 'published', 'author', 'copyright',
            'updated', 'tags', 'feeds']

    _pubparts = [('content', 'fragment')
                   ,('title', 'title')
                   ]

    def __repr__(self):
        dt = getattr(self, 'published', None)
        if dt is not None:
            dt = dt.strftime('%Y-%m-%d')
        return 'Post(%s-%s)' % (dt, getattr(self, 'slug', None))

    @property
    def _attributes(self):
        '''Attributes that identify a post object:

            year
               The year of publication.

            month
               The month of publication.

            day
               The day of publication.

            slug
               The `slug` attribute of the :class:`Post` object.

        .. doctest::

           >>> from datetime import datetime
           >>> pprint(Post(published=datetime(2009, 12, 31), slug='party')._attributes)
           {'day': 31, 'month': 12, 'slug': 'party', 'year': 2009}

        '''
        # pylint: disable-msg=E1101
        return {'year': self.published.year
               ,'month': self.published.month
               ,'day': self.published.day
               ,'slug': self.slug
               }


class PostParser(parsers.RstParser):
    '''Parse all posts matching ``[0-9]{4}-[0-9]{2}-[0-9]{2}-[-a-zA-Z0-9_]+``.

    .. doctest::
       :hide:

       >>> environment['log'] = get_logger()

    .. doctest::

       >>> pp = PostParser(environment, objects)
       >>> environment, objects, (parser,) = pp(environment, objects)
       >>> pprint(parser(environment, objects)) #doctest: +ELLIPSIS
       ({...},
        {'post': [Post(2009-12-31-party),
                  Post(2010-01-01-newyear),
                  Post(2010-02-01-newmonth),
                  Post(2010-02-02-newday),
                  Post(2010-02-02-newday2)]},
        [])

    '''

    type = 'post'
    paths = '^[0-9]{4}-[0-9]{2}-[0-9]{2}-[-a-zA-Z0-9_]+\.rst$'
    cls = Post

    @decorators.in_environment('settings')
    def root(self, environment):
        '''The directory under which all post objects reside.
        '''
        settings = environment['settings']
        return os.path.join(settings.CONTENT_ROOT, settings.POSTS_SUBDIR)

    def rstparse(self, environment, objects, path, pieces):
        '''Use the parsed rst document to construct the necessary objects.
        '''
        timezone = pytz.timezone(pieces['metadata'].get('timezone', 'UTC'))
        # Set times
        time = pieces['metadata'].get('time', datetime.time(0, 0, 0))
        pubdate = strptime(os.path.basename(path)[:10], ['%Y-%m-%d'])
        published = datetime.datetime.combine(pubdate.date(), time)
        updated = pieces['metadata'].get('updated', published)
        # Other attrs
        attrs = {}
        attrs['slug'] = unicode(path[11:-4])
        attrs['author'] = pieces['metadata'].get('author', '')
        attrs['copyright'] = pieces['metadata'].get('copyright', '')
        attrs['tags'] = pieces['metadata'].get('tags', [])
        attrs['feeds'] = pieces['metadata'].get('feeds', [])
        attrs['published'] = timezone.localize(published)
        attrs['updated'] = timezone.localize(updated)
        attrs['_pub'] = pieces['pub']
        objects[self.type].append(self.cls(**attrs))


def _setup(test):
    '''Setup the tests.
    '''
    from pysettings.settings import Settings
    from firmant import routing
    test.globs['settings'] = Settings({'CONTENT_ROOT': 'testdata/pristine'
                                      ,'POSTS_SUBDIR': 'posts'
                                      })
    urlmapper = routing.URLMapper('/path/to/output/dir', 'http://testurl/', [])
    test.globs['environment'] = {'settings': test.globs['settings']
                                ,'urlmapper': urlmapper
                                }
    test.globs['objects'] = {}
